==> nut-17.0.tar.gz: nut-17.0/fltk/AnalyzeMeals.cc <==
/* Class AnalyzeMeals */

/*
    NUT nutrition software 
    Copyright (C) 1996-2011 by Jim Jozwiak.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "Nut.h"
#include "AnalyzeMeals.h"
#include "AddToMealWidget.h"
#include "PrintMenus.h"
#include "anameal.h"
#include "options.h"
#include <FL/Fl_Pack.H>
#include <FL/Fl.H>
#include <string.h>

void food_suggestions_cb(Fl_Widget *which, void *who)
{
AnalyzeMeals *us_right_now = (AnalyzeMeals *)who;
us_right_now->food_suggestions();
}

void reset_calories_cb(Fl_Widget *which, void *who)
{
AnalyzeMeals *us_right_now = (AnalyzeMeals *)who;
which->deactivate();
us_right_now->reset_calories_gui();
which->activate();
}

void reset_macronutrients_cb(Fl_Widget *which, void *who)
{
AnalyzeMeals *us_right_now = (AnalyzeMeals *)who;
which->deactivate();
us_right_now->reset_macronutrients_gui();
which->activate();
}

void reset_everything_cb(Fl_Widget *which, void *who)
{
AnalyzeMeals *us_right_now = (AnalyzeMeals *)who;
which->deactivate();
us_right_now->reset_everything_gui();
which->activate();
}

void restore_defaults_cb(Fl_Widget *which, void *who)
{
AnalyzeMeals *us_right_now = (AnalyzeMeals *)who;
which->deactivate();
us_right_now->restore_defaults_gui();
which->activate();
}

AnalyzeMeals::AnalyzeMeals (int x, int y, int w, int h, Fl_Color widgetcolor) : Fl_Wizard (x, y, w, h)
{
int i;
for (i = 0; i < NUTRIENT_COUNT; i++) blank.nutrient[i] = NoData;
for (i = 0; i < NUTRIENT_COUNT; i++) zero.nutrient[i] = 0.00000000000001;
specialheader = 0;
mealroot = &meal_root;
meal_ptr_origin = &mealroot;
defanal = &(options.defanalanal);
this->label("Analyze Meals");
this->labelsize(FontSize);
this->labeltype(FL_ENGRAVED_LABEL);
nbw = new NutButtonWidget(x, y, w, h, widgetcolor, &foodwork);
this->end();
fs  = new FoodSuggestions(x, y, w, h, widgetcolor, this, &foodwork, "");
this->add(fs);

Fl_Pack *vpack;
for (i = 0; i < MAX_SCREEN; i++)
 {
 vpack = new Fl_Pack(x, y, w, 6*h/22);
  {
  new Fl_Box(x, y, w, h/22);
  }
  {
  if (i == 0)
   {
   Fl_Pack *hpack = new Fl_Pack(x, y, w, h/22);
   hpack->type(Fl_Pack::HORIZONTAL);
   hpack->spacing(w/96);
   new Fl_Box(x, y, w/36, h/22);
   Fl_Button *fsb = new Fl_Button(x, y, w/6+w/96, h/22, "Food Suggestions");
   Fl_Button *rcb = new Fl_Button(x, y, w/6+w/96, h/22, "Reset Calories");
   Fl_Button *rmb = new Fl_Button(x, y, w/6+w/96, h/22, "Reset Macronutrients");
   Fl_Button *reb = new Fl_Button(x, y, w/6+w/96, h/22, "Reset Everything");
   Fl_Button *rdb = new Fl_Button(x, y, w/6+w/96, h/22, "Restore Defaults");
   rcb->labelsize(FontSize);
   rmb->labelsize(FontSize);
   reb->labelsize(FontSize);
   rdb->labelsize(FontSize);
   fsb->labelsize(FontSize);
   rcb->clear_visible_focus();
   rmb->clear_visible_focus();
   reb->clear_visible_focus();
   rdb->clear_visible_focus();
   fsb->clear_visible_focus();
   rcb->color(fl_lighter(FL_BLUE));
   rmb->color(fl_lighter(FL_BLUE));
   reb->color(fl_lighter(FL_BLUE));
   rdb->color(fl_lighter(FL_BLUE));
   fsb->color(FL_GREEN);
   rcb->labelcolor(FL_YELLOW);
   rmb->labelcolor(FL_YELLOW);
   reb->labelcolor(FL_YELLOW);
   rdb->labelcolor(FL_YELLOW);
   rcb->callback(reset_calories_cb,this);
   rmb->callback(reset_macronutrients_cb,this);
   reb->callback(reset_everything_cb,this);
   rdb->callback(restore_defaults_cb,this);
   fsb->callback(food_suggestions_cb,this);
   hpack->end();
   }
  else new Fl_Box(x, y, w, h/22);
  }
 new Fl_Box(x, y, w, h/33);
 Fl_Pack *hpack = new Fl_Pack(x, y, w, 5*h/88-6*FontSize/16);
 hpack->type(Fl_Pack::HORIZONTAL);
  {
  here_are[i] = new Fl_Box(x, y, 2*w/3, h/44);
  here_are[i]->align(FL_ALIGN_INSIDE|FL_ALIGN_RIGHT);
  here_are[i]->labelsize(FontSize);
  meal_value_output[i] = new MealCountValueOutput(x, y, w/18, h/44, this, widgetcolor);
  meal_meals[i] = new Fl_Box(x, y, 3*w/12, h/44);
  meal_meals[i]->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT);
  meal_meals[i]->labelsize(FontSize);
  }
 hpack->end();
 {
 Fl_Pack *o = new Fl_Pack(x, y, w, h/22);
 o->type(Fl_Pack::HORIZONTAL);
  {
  Fl_Pack *o = new Fl_Pack(x, y, 2*w/3, 3*h/44);
   {
   new Fl_Box(x, y, w, h/44);
   }
   {
   Fl_Pack *o = new Fl_Pack(x, y, 2*w/3, h/22);
   o->type(Fl_Pack::HORIZONTAL);
   new Fl_Box(x, y, 3*w/36, h/22);
   ptm[i] = new PrintMenus(x, y, 2*w/18, h/22, widgetcolor);
   ptm[i]->off();
   new Fl_Box(x, y, 5*w/36, h/22);
   meal_enum[i] = new Fl_Box(x, y, w/3, h/22);
   meal_enum[i]->labelsize(FontSize);                 
   o->end();
   }
  o->end();
  }
 new Fl_Box(x,y,19*w/96,h/22);
 btm[i] = new BackToMenu(x, y, w/8, h/22, widgetcolor);
 btm[i]->off();
 o->end();
 }
 vpack->end();
 nbw->wizard[i][0]->add(vpack);
 }
update_am();
}

void AnalyzeMeals::update_am(void)
{
int meals, count, meal = 0;
char meal_date[9];
mealcount = meal_count(*meal_ptr_origin);
for (int i = 0; i < MAX_SCREEN; i++) meal_value_output[i]->maximum(mealcount);
if (mealcount > 0) for (int i = 0; i < MAX_SCREEN; i++) meal_value_output[i]->minimum(1);
else if (mealcount == 0) for (int i = 0; i < MAX_SCREEN; i++) meal_value_output[i]->minimum(0);
max = mealcount;
if (*defanal > 0 && *defanal < mealcount) max = *defanal;
if (mealcount > 0) foodwork = blank;
else foodwork = zero;
meal_ptr = *meal_ptr_origin;
strcpy(meal_date,"");
if (max > mealcount) max = mealcount;
meals = 0;
while (meal_ptr->next != NULL && meals <= max)
 {
 meal_ptr = meal_ptr->next;
 if (strcmp(meal_date,meal_ptr->meal_date) != 0 || meal != meal_ptr->meal)
  {
  strcpy(meal_date,meal_ptr->meal_date);
  meal = meal_ptr->meal;
  meals++;
  }
 if (meals > max) break;
 for (count = 0; count < NUTRIENT_COUNT; count++) foodwork.nutrient[count] += options.mealsperday * meal_ptr->grams / 100 * FoodIndex[meal_ptr->food_no]->nutrient[count] / (float) max;
 }
update_var_elements();
nbw->update();
if (nbw->visible()) nbw->redraw();
Fl::flush();
}

void AnalyzeMeals::update_am_dv_change_only(void)
{
update_var_elements();
nbw->update();
if (nbw->visible()) nbw->redraw();
Fl::flush();
}

void AnalyzeMeals::update_var_elements(void)
{
for (int i = 0; i < MAX_SCREEN; i++)
 {
 meal_value_output[i]->value(max);
 if (i == 0 && !options.custom) here_are[i]->label("Here are \"Daily Value\" average percentages for your previous");
 else if (i == 0 && options.custom) here_are[i]->label("Here are customized DV average percentages for your previous");
 else if (i != 0) here_are[i]->label("Here are average daily nutrient levels for your previous");
 if (max > 1 && max == mealcount) sprintf(mebuf," -> Meals  %s / %d  through  %s / %d\n\n",meal_ptr->meal_date,meal_ptr->meal,(*meal_ptr_origin)->next->meal_date,(*meal_ptr_origin)->next->meal);
 if (max > 1 && max == mealcount) sprintf(mebuf," -> Meals  %s / %d  through  %s / %d\n\n",meal_ptr->meal_date,meal_ptr->meal,(*meal_ptr_origin)->next->meal_date,(*meal_ptr_origin)->next->meal);
else if (max > 1 && max != mealcount) sprintf(mebuf," -> Meals  %s / %d  through  %s / %d\n\n",prev_meal(meal_ptr)->meal_date,prev_meal(meal_ptr)->meal,(*meal_ptr_origin)->next->meal_date,(*meal_ptr_origin)->next->meal);
 else if (max == 1) sprintf(mebuf," -> Meal  %s / %d\n\n",(*meal_ptr_origin)->next->meal_date,(*meal_ptr_origin)->next->meal);
 else if (max == 0) sprintf(mebuf," ");
 meal_enum[i]->label(mebuf);
 if (max != 1) meal_meals[i]->label("meals.");
 else meal_meals[i]->label("meal.");
 }
}

void AnalyzeMeals::reset_calories_gui(void)
{
if (max == 0) return;
reset_calories(&foodwork);
dv_change();
} 

void AnalyzeMeals::reset_macronutrients_gui(void)
{
if (max == 0) return;
reset_macronutrients(&foodwork);
dv_change();
}

void AnalyzeMeals::restore_defaults_gui(void)
{
if (max == 0 || options.screen != 0) return;
restore_defaults(0,&foodwork);
dv_change();
}

void AnalyzeMeals::food_suggestions(void)
{
this->next();
fs->update(max);
}

void AnalyzeMeals::reset_everything_gui(void)
{ 
if (max == 0) return;
reset_everything(&foodwork);
dv_change();
}

void AnalyzeMeals::meal_value_output_change(double newval)
{
int nv = int(newval);
*defanal = nv;
for (int i = 0; i < MAX_SCREEN; i++) meal_value_output[i]->value(newval);
update_am();
defanal_change(this);
}

void AnalyzeMeals::tab_change(void)
{
fs->tab_change();                           
}

void AnalyzeMeals::back_to_nbw(void)
{
this->value(nbw);
}

void AnalyzeMeals::set_specialheader(RecordMeals *rm, struct meal **rmroot)
{                                   
specialheader = 1;
for (int i = 0; i < MAX_SCREEN; i++)
 {
 btm[i]->on(rm);
 ptm[i]->on(rm);
 }
defanal = &(options.defanalrec);
meal_ptr_origin = rmroot;
}

struct meal *AnalyzeMeals::tell_me_meal_root(void)
{
return *meal_ptr_origin;
}

==> nut-17.4.tar.gz: nut-17.4/fltk/AnalyzeMeals.cc <==
/* Class AnalyzeMeals */

/*
    NUT nutrition software 
    Copyright (C) 1996-2011 by Jim Jozwiak.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "Nut.h"
#include "AnalyzeMeals.h"
#include "AddToMealWidget.h"
#include "PrintMenus.h"
#include "anameal.h"
#include "options.h"
#include <FL/Fl_Pack.H>
#include <FL/Fl.H>
#include <string.h>

void food_suggestions_cb(Fl_Widget *which, void *who)
{
AnalyzeMeals *us_right_now = (AnalyzeMeals *)who;
us_right_now->food_suggestions();
}

void reset_calories_cb(Fl_Widget *which, void *who)
{
AnalyzeMeals *us_right_now = (AnalyzeMeals *)who;
which->deactivate();
us_right_now->reset_calories_gui();
which->activate();
}

void reset_macronutrients_cb(Fl_Widget *which, void *who)
{
AnalyzeMeals *us_right_now = (AnalyzeMeals *)who;
which->deactivate();
us_right_now->reset_macronutrients_gui();
which->activate();
}

void reset_everything_cb(Fl_Widget *which, void *who)
{
AnalyzeMeals *us_right_now = (AnalyzeMeals *)who;
which->deactivate();
us_right_now->reset_everything_gui();
which->activate();
}

void restore_defaults_cb(Fl_Widget *which, void *who)
{
AnalyzeMeals *us_right_now = (AnalyzeMeals *)who;
which->deactivate();
us_right_now->restore_defaults_gui();
which->activate();
}

AnalyzeMeals::AnalyzeMeals (int x, int y, int w, int h, Fl_Color widgetcolor) : Fl_Wizard (x, y, w, h)
{
int i;
for (i = 0; i < NUTRIENT_COUNT; i++) blank.nutrient[i] = NoData;
for (i = 0; i < NUTRIENT_COUNT; i++) zero.nutrient[i] = 0.00000000000001;
specialheader = 0;
mealroot = &meal_root;
meal_ptr_origin = &mealroot;
defanal = &(options.defanalanal);
this->label("Analyze Meals");
this->labeltype(FL_ENGRAVED_LABEL);
nbw = new NutButtonWidget(x, y, w, h, widgetcolor, &foodwork);
this->end();
fs  = new FoodSuggestions(x, y, w, h, widgetcolor, this, &foodwork, "");
this->add(fs);

Fl_Pack *vpack;
for (i = 0; i < MAX_SCREEN; i++)
 {
 vpack = new Fl_Pack(x, y, w, 6*h/22);
  {
  new Fl_Box(x, y, w, h/22);
  }
  {
  if (i == 0)
   {
   Fl_Pack *hpack = new Fl_Pack(x, y, w, h/22);
   hpack->type(Fl_Pack::HORIZONTAL);
   hpack->spacing(w/96);
   new Fl_Box(x, y, w/36, h/22);
   Fl_Button *fsb = new Fl_Button(x, y, w/6+w/96, h/22, "Food Suggestions");
   Fl_Button *rcb = new Fl_Button(x, y, w/6+w/96, h/22, "Reset Calories");
   Fl_Button *rmb = new Fl_Button(x, y, w/6+w/96, h/22, "Reset Macronutrients");
   Fl_Button *reb = new Fl_Button(x, y, w/6+w/96, h/22, "Reset Everything");
   Fl_Button *rdb = new Fl_Button(x, y, w/6+w/96, h/22, "Restore Defaults");
   rcb->clear_visible_focus();
   rmb->clear_visible_focus();
   reb->clear_visible_focus();
   rdb->clear_visible_focus();
   fsb->clear_visible_focus();
   rcb->color(fl_lighter(FL_BLUE));
   rmb->color(fl_lighter(FL_BLUE));
   reb->color(fl_lighter(FL_BLUE));
   rdb->color(fl_lighter(FL_BLUE));
   fsb->color(FL_GREEN);
   rcb->labelcolor(FL_YELLOW);
   rmb->labelcolor(FL_YELLOW);
   reb->labelcolor(FL_YELLOW);
   rdb->labelcolor(FL_YELLOW);
   rcb->callback(reset_calories_cb,this);
   rmb->callback(reset_macronutrients_cb,this);
   reb->callback(reset_everything_cb,this);
   rdb->callback(restore_defaults_cb,this);
   fsb->callback(food_suggestions_cb,this);
   hpack->end();
   }
  else new Fl_Box(x, y, w, h/22);
  }
 new Fl_Box(x, y, w, h/33);
 Fl_Pack *hpack = new Fl_Pack(x, y, w, 5*h/88-6*FL_NORMAL_SIZE/16);
 hpack->type(Fl_Pack::HORIZONTAL);
  {
  here_are[i] = new Fl_Box(x, y, 2*w/3, h/44);
  here_are[i]->align(FL_ALIGN_INSIDE|FL_ALIGN_RIGHT);
  meal_value_output[i] = new MealCountValueOutput(x, y, w/18, h/44, this, widgetcolor);
  meal_meals[i] = new Fl_Box(x, y, 3*w/12, h/44);
  meal_meals[i]->align(FL_ALIGN_INSIDE|FL_ALIGN_LEFT);
  }
 hpack->end();
 {
 Fl_Pack *o = new Fl_Pack(x, y, w, h/22);
 o->type(Fl_Pack::HORIZONTAL);
  {
  Fl_Pack *o = new Fl_Pack(x, y, 2*w/3, 3*h/44);
   {
   new Fl_Box(x, y, w, h/44);
   }
   {
   Fl_Pack *o = new Fl_Pack(x, y, 2*w/3, h/22);
   o->type(Fl_Pack::HORIZONTAL);
   new Fl_Box(x, y, 3*w/36, h/22);
   ptm[i] = new PrintMenus(x, y, 2*w/18, h/22, widgetcolor);
   ptm[i]->off();
   new Fl_Box(x, y, 5*w/36, h/22);
   meal_enum[i] = new Fl_Box(x, y, w/3, h/22);
   o->end();
   }
  o->end();
  }
 new Fl_Box(x,y,19*w/96,h/22);
 btm[i] = new BackToMenu(x, y, w/8, h/22, widgetcolor);
 btm[i]->off();
 o->end();
 }
 vpack->end();
 nbw->wizard[i][0]->add(vpack);
 }
update_am();
}

void AnalyzeMeals::update_am(void)
{
int meals, count, meal = 0;
char meal_date[9];
mealcount = meal_count(*meal_ptr_origin);
for (int i = 0; i < MAX_SCREEN; i++) meal_value_output[i]->maximum(mealcount);
if (mealcount > 0) for (int i = 0; i < MAX_SCREEN; i++) meal_value_output[i]->minimum(1);
else if (mealcount == 0) for (int i = 0; i < MAX_SCREEN; i++) meal_value_output[i]->minimum(0);
max = mealcount;
if (*defanal > 0 && *defanal < mealcount) max = *defanal;
if (mealcount > 0) foodwork = blank;
else foodwork = zero;
meal_ptr = *meal_ptr_origin;
strcpy(meal_date,"");
if (max > mealcount) max = mealcount;
meals = 0;
while (meal_ptr->next != NULL && meals <= max)
 {
 meal_ptr = meal_ptr->next;
 if (strcmp(meal_date,meal_ptr->meal_date) != 0 || meal != meal_ptr->meal)
  {
  strcpy(meal_date,meal_ptr->meal_date);
  meal = meal_ptr->meal;
  meals++;
  }
 if (meals > max) break;
 for (count = 0; count < NUTRIENT_COUNT; count++) foodwork.nutrient[count] += options.mealsperday * meal_ptr->grams / 100 * FoodIndex[meal_ptr->food_no]->nutrient[count] / (float) max;
 }
update_var_elements();
nbw->update();
if (nbw->visible()) nbw->redraw();
Fl::flush();
}

void AnalyzeMeals::update_am_dv_change_only(void)
{
update_var_elements();
nbw->update();
if (nbw->visible()) nbw->redraw();
Fl::flush();
}

void AnalyzeMeals::update_var_elements(void)
{
for (int i = 0; i < MAX_SCREEN; i++)
 {
 meal_value_output[i]->value(max);
 if (i == 0 && !options.custom) here_are[i]->label("Here are \"Daily Value\" average percentages for your previous");
 else if (i == 0 && options.custom) here_are[i]->label("Here are customized DV average percentages for your previous");
 else if (i != 0) here_are[i]->label("Here are average daily nutrient levels for your previous");
 if (max > 1 && max == mealcount) sprintf(mebuf," -> Meals  %s / %d  through  %s / %d\n\n",meal_ptr->meal_date,meal_ptr->meal,(*meal_ptr_origin)->next->meal_date,(*meal_ptr_origin)->next->meal);
 if (max > 1 && max == mealcount) sprintf(mebuf," -> Meals  %s / %d  through  %s / %d\n\n",meal_ptr->meal_date,meal_ptr->meal,(*meal_ptr_origin)->next->meal_date,(*meal_ptr_origin)->next->meal);
else if (max > 1 && max != mealcount) sprintf(mebuf," -> Meals  %s / %d  through  %s / %d\n\n",prev_meal(meal_ptr)->meal_date,prev_meal(meal_ptr)->meal,(*meal_ptr_origin)->next->meal_date,(*meal_ptr_origin)->next->meal);
 else if (max == 1) sprintf(mebuf," -> Meal  %s / %d\n\n",(*meal_ptr_origin)->next->meal_date,(*meal_ptr_origin)->next->meal);
 else if (max == 0) sprintf(mebuf," ");
 meal_enum[i]->label(mebuf);
 if (max != 1) meal_meals[i]->label("meals.");
 else meal_meals[i]->label("meal.");
 }
}

void AnalyzeMeals::reset_calories_gui(void)
{
if (max == 0) return;
reset_calories(&foodwork);
dv_change();
} 

void AnalyzeMeals::reset_macronutrients_gui(void)
{
if (max == 0) return;
reset_macronutrients(&foodwork);
dv_change();
}

void AnalyzeMeals::restore_defaults_gui(void)
{
if (max == 0 || options.screen != 0) return;
restore_defaults(0,&foodwork);
dv_change();
}

void AnalyzeMeals::food_suggestions(void)
{
this->next();
fs->update(max);
}

void AnalyzeMeals::reset_everything_gui(void)
{ 
if (max == 0) return;
reset_everything(&foodwork);
dv_change();
}

void AnalyzeMeals::meal_value_output_change(double newval)
{
int nv = int(newval);
*defanal = nv;
for (int i = 0; i < MAX_SCREEN; i++) meal_value_output[i]->value(newval);
update_am();
defanal_change(this);
}

void AnalyzeMeals::tab_change(void)
{
fs->tab_change();                           
}

void AnalyzeMeals::back_to_nbw(void)
{
this->value(nbw);
}

void AnalyzeMeals::set_specialheader(RecordMeals *rm, struct meal **rmroot)
{                                   
specialheader = 1;
for (int i = 0; i < MAX_SCREEN; i++)
 {
 btm[i]->on(rm);
 ptm[i]->on(rm);
 }
defanal = &(options.defanalrec);
meal_ptr_origin = rmroot;
}

struct meal *AnalyzeMeals::tell_me_meal_root(void)
{
return *meal_ptr_origin;
}
